@@ -11,10 +11,13 @@
 #include <QDir>
 #include <QSettings>
 #include <QStandardPaths>
+#include <QLoggingCategory>
+
+Q_DECLARE_LOGGING_CATEGORY(logModels)
 
 ItemArrangementProxyModel::~ItemArrangementProxyModel()
 {
-
+    qCDebug(logModels) << "Destroying ItemArrangementProxyModel";
 }
 
 int ItemArrangementProxyModel::pageCount(int folderId) const
@@ -24,11 +27,17 @@ int ItemArrangementProxyModel::pageCount(int folderId) const
     QString fullId("internal/folders/" + QString::number(folderId));
     Q_ASSERT(m_folders.contains(fullId));
 
-    return m_folders.value(fullId)->pageCount();
+    if (auto itemPage = m_folders.value(fullId); itemPage) {
+        return itemPage->pageCount();
+    } else {
+        qWarning() << "itemPage is null, return 0. fullId is" << fullId;
+        return 0;
+    }
 }
 
 void ItemArrangementProxyModel::updateFolderName(int folderId, const QString &name)
 {
+    qCInfo(logModels) << "Updating folder name:" << folderId << "to" << name;
     ItemsPage * folder = folderById(folderId);
     folder->setName(name);
 
@@ -39,12 +48,43 @@ void ItemArrangementProxyModel::updateFolderName(int folderId, const QString &na
     saveItemArrangementToUserData();
 }
 
+void ItemArrangementProxyModel::bringToFront(const QString & id)
+{
+    std::tuple<int, int, int> origPos = findItem(id);
+
+    // can only bring top-level item to front
+    if (std::get<0>(origPos) != 0) return;
+
+    // already at front
+    if (std::get<1>(origPos) == 0 && std::get<2>(origPos) == 0) return;
+
+    m_topLevel->moveItemPosition(std::get<1>(origPos), std::get<2>(origPos), 0, 0, false);
+
+    saveItemArrangementToUserData();
+
+    // Lazy solution, just notify the view that all rows and its roles are changed so they need to be updated.
+    emit dataChanged(index(0, 0), index(rowCount() - 1, 0), {
+        PageRole, IndexInPageRole, FolderIdNumberRole, IconsNameRole
+    });
+}
+
 void ItemArrangementProxyModel::commitDndOperation(const QString &dragId, const QString &dropId, const DndOperation op, int pageHint)
 {
-    if (dragId == dropId) return;
+    
+    if (dragId == dropId) {
+        qCDebug(logModels) << "Drag and drop IDs are the same, returning early";
+        return;
+    }
 
     std::tuple<int, int, int> dragOrigPos = findItem(dragId);
     std::tuple<int, int, int> dropOrigPos = findItem(dropId);
+    qCDebug(logModels) << "Drop position:" << std::get<0>(dropOrigPos) << std::get<1>(dropOrigPos) << std::get<2>(dropOrigPos);
+
+    Q_ASSERT(std::get<0>(dragOrigPos) != -1);
+    if (std::get<0>(dragOrigPos) == -1) {
+        qCWarning(logModels) << "Cannot find drag item" << dragId << "in current item arrangement";
+        return;
+    }
 
     if (op != DndOperation::DndJoin) {
         // move to dropId's front or back
@@ -55,14 +95,18 @@ void ItemArrangementProxyModel::commitDndOperation(const QString &dragId, const
             const int dropOrigPage = std::get<1>(dropOrigPos);
             const int fromIndex = std::get<2>(dragOrigPos);
             const int toIndex = std::get<2>(dropOrigPos);
-            qDebug() << "dragOrigPage" << dragOrigPage << "dropOrigPage" << dropOrigPage << "fromIndex" << fromIndex << "toIndex" << toIndex << "op" << (op == DndOperation::DndAppend);
-            folder->moveItemPosition(dragOrigPage, fromIndex, dropOrigPage, toIndex, op == DndOperation::DndAppend);
+            bool isAppend = (op == DndOperation::DndAppend);
+            qCDebug(logModels) << "dragOrigPage" << dragOrigPage << "dropOrigPage" << dropOrigPage 
+                              << "fromIndex" << fromIndex << "toIndex" << toIndex << "isAppend" << isAppend;
+            folder->moveItemPosition(dragOrigPage, fromIndex, dropOrigPage, toIndex, isAppend);
         } else {
             // different folder item arrangement
             ItemsPage * srcFolder = folderById(std::get<0>(dragOrigPos));
             ItemsPage * dstFolder = folderById(std::get<0>(dropOrigPos));
+            qCDebug(logModels) << "Removing item from source folder";
             srcFolder->removeItem(dragId);
             if (srcFolder->pageCount() == 0 && srcFolder != dstFolder) {
+                qCDebug(logModels) << "Source folder is empty and different from destination, removing it";
                 removeFolder(QString::number(std::get<0>(dragOrigPos)));
             }
             dstFolder->insertItem(dragId, std::get<1>(dropOrigPos), std::get<2>(dropOrigPos));
@@ -74,29 +118,60 @@ void ItemArrangementProxyModel::commitDndOperation(const QString &dragId, const
         // the source item will be inside a new folder anyway.
         const int srcFolderId = std::get<0>(dragOrigPos);
         ItemsPage * srcFolder = folderById(srcFolderId);
-        srcFolder->removeItem(dragId);
+        qCDebug(logModels) << "Source folder ID:" << srcFolderId;
 
         if (dropId.startsWith("internal/folders/")) {
+            qCDebug(logModels) << "Drop into existing folder:" << dropId;
             // drop into existing folder
             const int dropOrigFolder = QStringView{dropId}.mid(17).toInt();
             ItemsPage * dstFolder = folderById(dropOrigFolder);
-            if (srcFolder->pageCount() == 0 && srcFolder != dstFolder) {
+            const int fromPage = std::get<1>(dragOrigPos);
+            const int &toPage = pageHint;
+            qCDebug(logModels) << "From page:" << fromPage << "to page:" << toPage;
+            
+            if (srcFolder == dstFolder) {
+                bool isSingleItem = (srcFolder->itemCount() == 1);
+                bool isSingleItemOnSamePage = (fromPage == toPage && srcFolder->itemCount(fromPage) == 1);
+                qCDebug(logModels) << "Same folder check - isSingleItem:" << isSingleItem 
+                                  << "isSingleItemOnSamePage:" << isSingleItemOnSamePage;
+                
+                if (isSingleItem || isSingleItemOnSamePage) {
+                    qCDebug(logModels) << "DnD the only item to the same page, returning";
+                    return;
+                }
+            }
+
+            // hold the empty page avoid access out of page range !
+            srcFolder->removeItem(dragId, false);
+            if (srcFolder->itemCount() == 0 && srcFolder != dstFolder) {
+                qCDebug(logModels) << "Source folder is empty and different, removing folder";
                 removeFolder(QString::number(srcFolderId));
             }
-            dstFolder->appendItem(dragId, pageHint);
+            dstFolder->insertItemToPage(dragId, pageHint);
+
+            // clear empty page
+            qCDebug(logModels) << "Clearing empty pages from source folder";
+            srcFolder->removeEmptyPages();
         } else {
+            srcFolder->removeItem(dragId);
             // make a new folder, move two items into the folder
             QString dstFolderId = findAvailableFolderId();
             ItemsPage * dstFolder = createFolder(dstFolderId);
-            dstFolder->appendPage({dragId, dropId});
+            qCDebug(logModels) << "Appending items to new folder page:" << dropId << dragId;
+            dstFolder->appendPage({dropId, dragId});
             AppItem * dropItem = AppsModel::instance().itemFromDesktopId(dropId);
-            AppItem::DDECategories dropCategories = AppItem::DDECategories(CategoryUtils::parseBestMatchedCategory(dropItem->categories()));
-            dstFolder->setName("internal/category/" + QString::number(dropCategories));
+            if (dropItem) {
+                AppItem::DDECategories dropCategories = AppItem::DDECategories(CategoryUtils::parseBestMatchedCategory(dropItem->categories()));
+                QString folderName = "internal/category/" + QString::number(dropCategories);
+                qCDebug(logModels) << "New folder name:" << folderName;
+                dstFolder->setName(folderName);
+            }
             if (srcFolder->pageCount() == 0 && srcFolder != m_topLevel) {
+                qCDebug(logModels) << "Source folder is empty and not top level, removing it";
                 removeFolder(QString::number(srcFolderId));
             }
-            m_topLevel->removeItem(dropId);
             m_topLevel->insertItem(dstFolderId, std::get<1>(dropOrigPos), std::get<2>(dropOrigPos));
+            m_topLevel->removeItem(dropId);
         }
     }
 
@@ -108,6 +183,32 @@ void ItemArrangementProxyModel::commitDndOperation(const QString &dragId, const
     });
 }
 
+// return new empty page index
+int ItemArrangementProxyModel::creatEmptyPage(int folderId) const
+{
+    if(folderId == 0){
+        m_topLevel->appendEmptyPage();
+        return m_topLevel->pageCount() - 1;
+    }
+    QString fullId("internal/folders/" + QString::number(folderId));
+    Q_ASSERT(m_folders.contains(fullId));
+
+    if (auto itemPage = m_folders.value(fullId); itemPage) {
+        itemPage->appendEmptyPage();
+        const auto& result = itemPage->pageCount() - 1;
+        qCInfo(logModels) << "Created empty page at index:" << result << "for folder:" << fullId;
+        return result;
+    } else {
+        qCWarning(logModels) << "itemPage create empty page false, return 0. fullId is" << fullId;
+        return 0;
+    }
+}
+
+void ItemArrangementProxyModel::removeEmptyPage() const
+{
+    m_topLevel->removeEmptyPages();
+}
+
 QVariant ItemArrangementProxyModel::data(const QModelIndex &index, int role) const
 {
     int idx = index.row() - AppsModel::instance().rowCount();
@@ -117,7 +218,7 @@ QVariant ItemArrangementProxyModel::data(const QModelIndex &index, int role) con
         // regular applications, not a folder
         QString id(data(index, AppItem::DesktopIdRole).toString());
         if (id.isEmpty() || id.contains("internal")) {
-            qDebug() << id << index << index.row() << AppsModel::instance().rowCount() << role;
+            qCWarning(logModels) << "Invalid or internal ID:" << id << "index:" << index << "row:" << index.row() << "role:" << role;
         }
         int folder, page, idx;
         std::tie(folder, page, idx) = findItem(id);
@@ -131,6 +232,8 @@ QVariant ItemArrangementProxyModel::data(const QModelIndex &index, int role) con
                 return folder;
             case IconsNameRole:
                 return QVariant();
+            case ItemTypeRole:
+                return AppItemType;
         }
     } else {
         // a folder
@@ -146,6 +249,8 @@ QVariant ItemArrangementProxyModel::data(const QModelIndex &index, int role) con
                 return m_folders.value(id)->name();
             case AppItem::DesktopIdRole:
                 return id;
+            case AppItem::IsAutoStartRole:
+                return false;
             case PageRole:
                 return page;
             case IndexInPageRole:
@@ -163,9 +268,10 @@ QVariant ItemArrangementProxyModel::data(const QModelIndex &index, int role) con
                 }
                 return icons;//QStringList({"deepin-music"});
             }
+            case ItemTypeRole:
+                return FolderItemType;
         }
     }
-
     return QConcatenateTablesProxyModel::data(index, role);
 }
 
@@ -173,12 +279,13 @@ QHash<int, QByteArray> ItemArrangementProxyModel::roleNames() const
 {
     auto existingRoleNames = AppsModel::instance().roleNames();
     existingRoleNames.insert(IconsNameRole, QByteArrayLiteral("folderIcons"));
+    existingRoleNames.insert(ItemTypeRole, QByteArrayLiteral("itemType"));
     return existingRoleNames;
 }
 
 ItemArrangementProxyModel::ItemArrangementProxyModel(QObject *parent)
     : QConcatenateTablesProxyModel(parent)
-    , m_topLevel(new ItemsPage(7 * 4, this))
+    , m_topLevel(new ItemsPage(8 * 4, this))
 {
     m_folderModel.setItemRoleNames(AppsModel::instance().roleNames());
 
@@ -199,8 +306,8 @@ ItemArrangementProxyModel::ItemArrangementProxyModel(QObject *parent)
 
 void ItemArrangementProxyModel::loadItemArrangementFromUserData()
 {
-    const QString arrangementSettingBasePath(QStandardPaths::writableLocation(QStandardPaths::AppConfigLocation));
-    const QString arrangementSettingPath(QDir(arrangementSettingBasePath).absoluteFilePath("item-arrangement.ini"));
+    const QString arrangementSettingBasePath(QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation));
+    const QString arrangementSettingPath(QDir(arrangementSettingBasePath).absoluteFilePath("deepin/dde-launchpad/item-arrangement.ini"));
     QSettings itemArrangementSettings(arrangementSettingPath, QSettings::NativeFormat);
 
     itemArrangementSettings.beginGroup("fullscreen");
@@ -212,7 +319,7 @@ void ItemArrangementProxyModel::loadItemArrangementFromUserData()
         int pageCount = itemArrangementSettings.value("pageCount", 0).toInt();
         bool isTopLevel = groupName == "toplevel";
 
-        qDebug() << groupName << folderName << pageCount;
+        qCDebug(logModels) << "Group details - name:" << groupName << "folder name:" << folderName << "page count:" << pageCount << "isTopLevel:" << isTopLevel;
 
         ItemsPage * page = isTopLevel ? m_topLevel : createFolder(groupName);
         page->setName(folderName);
@@ -228,8 +335,8 @@ void ItemArrangementProxyModel::loadItemArrangementFromUserData()
 
 void ItemArrangementProxyModel::saveItemArrangementToUserData()
 {
-    const QString arrangementSettingBasePath(QStandardPaths::writableLocation(QStandardPaths::AppConfigLocation));
-    const QString arrangementSettingPath(QDir(arrangementSettingBasePath).absoluteFilePath("item-arrangement.ini"));
+    const QString arrangementSettingBasePath(QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation));
+    const QString arrangementSettingPath(QDir(arrangementSettingBasePath).absoluteFilePath("deepin/dde-launchpad/item-arrangement.ini"));
     QSettings itemArrangementSettings(arrangementSettingPath, QSettings::NativeFormat);
     itemArrangementSettings.clear();
 
@@ -284,19 +391,44 @@ void ItemArrangementProxyModel::onSourceModelChanged()
     for (int i = 0; i < appsCount; i++) {
         QString desktopId(AppsModel::instance().data(AppsModel::instance().index(i, 0), AppItem::DesktopIdRole).toString());
         appDesktopIdSet.insert(desktopId);
-        int folder, page, idx;
+        int folder;
         std::tie(folder, std::ignore, std::ignore) = findItem(desktopId);
         // add all existing ones if they are not already in
         if (folder == -1) {
             findItem(desktopId);
-            m_topLevel->appendItem(desktopId);
+            // Find first available page with space
+            int targetPage = -1;
+            int targetIndex = -1;
+            
+            // Check each page for available space
+            for (int page = 0; page < m_topLevel->pageCount(); page++) {
+                int itemCount = m_topLevel->itemCount(page);
+                if (itemCount < m_topLevel->maxItemCountPerPage()) {
+                    targetPage = page;
+                    targetIndex = itemCount;
+                    break;
+                }
+            }
+            
+            if (targetPage != -1) {
+                // Add to first available page with space
+                qCDebug(logModels) << "Adding item to existing page" << targetPage << "at index" << targetIndex;
+                m_topLevel->insertItem(desktopId, targetPage, targetIndex);
+            } else {
+                // All pages are full, append to new page
+                qCDebug(logModels) << "All pages full, appending item to new page";
+                m_topLevel->appendItem(desktopId);
+            }
         }
     }
 
     m_topLevel->removeItemsNotIn(appDesktopIdSet);
-    for (int i = 0; i < m_folderModel.rowCount(); i++) {
+    for (int i = m_folderModel.rowCount() - 1; i >= 0 ; i--) {
         const QString & folderId = m_folderModel.index(i, 0).data(AppItem::DesktopIdRole).toString();
         m_folders.value(folderId)->removeItemsNotIn(appDesktopIdSet);
+        if (m_folders.value(folderId)->pageCount() == 0) {
+            removeFolder(QString(folderId).remove("internal/folders/"));
+        }
     }
 
     emit dataChanged(index(0, 0), index(rowCount() - 1, 0), {
@@ -328,6 +460,7 @@ QString ItemArrangementProxyModel::findAvailableFolderId()
     } while (m_folders.contains(fullId));
 
     Q_ASSERT(idNumber != 0); // 0 is reserved for top level.
+    qCDebug(logModels) << "Found available folder ID:" << fullId;
     return fullId;
 }
 
@@ -343,6 +476,11 @@ ItemsPage *ItemArrangementProxyModel::createFolder(const QString &id)
     folder->setData(fullId, AppItem::DesktopIdRole);
     m_folderModel.appendRow(folder);
 
+    connect(page, &ItemsPage::pageCountChanged, this, [this, fullId]() {
+        int folderId = QStringView{fullId}.mid(17).toInt();
+        emit folderPageCountChanged(folderId);
+    });
+
     return page;
 }
 
